==> geo-xform-1.1.4.tgz: package/lib/vrt.js <==
/* @ flow */
'use strict';

var fs = require('fs');
var Geojson = require('./geojson');
var _ = require('highland');
var EventEmitter = require('events').EventEmitter;
var util = require('util');
var lodash = require('lodash');
var FeatureParser = require('feature-parser');

function createStream(options) {
  var size = options.size || 5000;
  var watcher = new Watcher();
  var vrtPath = options.path + '/layer.vrt';
  var vrt = fs.createWriteStream(vrtPath);
  vrt.write('<OGRVRTDataSource>');
  var index = 0;
  var firstBatch = true;
  var input = _();
  input.pipe(FeatureParser.parse()).batch(size).each(function (batch) {
    if (firstBatch) {
      firstBatch = false;
      var properties = undefined;
      try {
        properties = sample(batch);
      } catch (e) {
        input.emit('log', { level: 'error', message: { error: 'Bad batch of geojson', batch: batch } });
        input.emit('error', e);
        return input.destroy();
      }
      input.emit('properties', properties);
    }
    var fileName = options.path + '/part.' + index + '.json';
    var writer = writeLayer(batch, fileName);
    watcher.watch(writer);
    vrt.write(metadata(fileName));
    index++;
  }).done(function () {
    vrt.write('</OGRVRTDataSource>');
    if (watcher.idle) input.emit('finish', vrtPath);else watcher.on('idle', function () {
      return input.emit('finish', vrtPath);
    });
  });
  return input;
}

function sample(batch) {
  var sample = lodash.find(function (string) {
    var feature = JSON.parse(string);
    if (feature.geometry && feature.geometry.type) return true;else return false;
  });
  sample = JSON.parse(sample || batch[0]);
  var geometry = sample.geometry ? sample.geometry.type : 'NONE';
  var fields = Object.keys(sample.properties);
  return { geometry: geometry, fields: fields };
}

function metadata(fileName) {
  return '<OGRVRTLayer name="OGRGeoJSON"><SrcDataSource>' + fileName + '</SrcDataSource></OGRVRTLayer>';
}

function writeLayer(batch, fileName) {
  var fileStream = fs.createWriteStream(fileName);
  return _(batch).pipe(Geojson.createStream()).pipe(fileStream);
}

// This object's job is to make sure that we don't close the stream until
// all the vrt parts have been fully written to disk
function Watcher() {
  this.writers = [];
}
util.inherits(Watcher, EventEmitter);

Watcher.prototype.watch = function (writer) {
  var _this = this;

  this.idle = false;
  this.writers.push(false);
  var index = this.writers.length - 1;
  writer.on('finish', function () {
    _this.writers[index] = true;
    // this will emit finish when all the outstanding writers come back
    if (_this.writers.indexOf(false) < 0) {
      _this.idle = true;
      _this.emit('idle');
    }
  });
};

module.exports = { createStream: createStream };

==> geo-xform-1.2.2.tgz: package/lib/vrt.js <==
/* @ flow */
'use strict';

var fs = require('fs');
var Geojson = require('./geojson');
var _ = require('highland');
var EventEmitter = require('events').EventEmitter;
var util = require('util');
var lodash = require('lodash');
var FeatureParser = require('feature-parser');

function createStream(options) {
  var size = options.size || 5000;
  var watcher = new Watcher();
  var vrtPath = options.path + '/layer.vrt';
  var vrt = fs.createWriteStream(vrtPath);
  vrt.write('<OGRVRTDataSource>');
  var index = 0;
  var firstBatch = true;
  var input = _();
  input.pipe(FeatureParser.parse()).batch(size).each(function (batch) {
    if (firstBatch) {
      firstBatch = false;
      var properties = undefined;
      try {
        properties = sample(batch);
      } catch (e) {
        input.emit('log', { level: 'error', message: { error: 'Bad batch of geojson', batch: batch } });
        input.emit('error', e);
        return input.destroy();
      }
      input.emit('properties', properties);
    }
    var fileName = options.path + '/part.' + index + '.json';
    var writer = writeLayer(batch, fileName);
    watcher.watch(writer);
    vrt.write(metadata(fileName));
    index++;
  }).done(function () {
    vrt.end('</OGRVRTDataSource>');
    if (watcher.idle) input.emit('finish', vrtPath);else watcher.on('idle', function () {
      return input.emit('finish', vrtPath);
    });
  });
  return input;
}

function sample(batch) {
  var sample = lodash.find(function (string) {
    var feature = JSON.parse(string);
    if (feature.geometry && feature.geometry.type) return true;else return false;
  });
  sample = JSON.parse(sample || batch[0]);
  var geometry = sample.geometry ? sample.geometry.type : 'NONE';
  var fields = Object.keys(sample.properties);
  return { geometry: geometry, fields: fields };
}

function metadata(fileName) {
  return '<OGRVRTLayer name="OGRGeoJSON"><SrcDataSource>' + fileName + '</SrcDataSource></OGRVRTLayer>';
}

function writeLayer(batch, fileName) {
  var fileStream = fs.createWriteStream(fileName);
  return _(batch).pipe(Geojson.createStream()).pipe(fileStream);
}

// This object's job is to make sure that we don't close the stream until
// all the vrt parts have been fully written to disk
function Watcher() {
  this.writers = [];
}
util.inherits(Watcher, EventEmitter);

Watcher.prototype.watch = function (writer) {
  var _this = this;

  this.idle = false;
  this.writers.push(false);
  var index = this.writers.length - 1;
  writer.on('finish', function () {
    _this.writers[index] = true;
    // this will emit finish when all the outstanding writers come back
    if (_this.writers.indexOf(false) < 0) {
      _this.idle = true;
      _this.emit('idle');
    }
  });
};

module.exports = { createStream: createStream };
